@@ -19,6 +19,8 @@ TAG_DEFINITIONS = {
 
 FONT_SCALES = ['small', 'normal', 'large', 'larger']
 
+LINK_ENDERS = ['\n', '\t', ' ', '.', ',', ';', ':']
+
 class GenericAction(object):
     def maybe_join(self, new_action):
         return False
@@ -375,7 +377,7 @@ class CheckBox(Gtk.CheckButton):
         Gtk.CheckButton.do_size_allocate(self, allocation)
 
         if self.event_window:
-            self.event_window.move(allocation.x, allocation.y)
+            self.event_window.move_resize(allocation.x, allocation.y, allocation.width, allocation.height)
 
 class NoteBuffer(Gtk.TextBuffer):
     # These values should not be modified directly.
@@ -427,6 +429,9 @@ class NoteBuffer(Gtk.TextBuffer):
 
     def set_view(self, view):
         def track_motion(v, event):
+            if self.view.get_modifier_mask(Gdk.ModifierIntent.EXTEND_SELECTION):
+                return Gdk.EVENT_PROPAGATE
+
             mouse_iter = self.view.get_iter_at_location(*self.view.window_to_buffer_coords(Gtk.TextWindowType.TEXT, event.x, event.y))[1]
             if mouse_iter.has_tag(self.get_tag_table().lookup('link')):
                 self.view.props.window.set_cursor(Gdk.Cursor.new_from_name(Gdk.Display.get_default(), 'pointer'))
@@ -631,11 +636,16 @@ class NoteBuffer(Gtk.TextBuffer):
 
                     action = CompositeAction(action, obj_action)
 
-            # location gets shifted by one when Gtk.TextBuffer.do_insert_text() gets called, and becomes invalidated by
-            # adding a check/bullet, so we need to put it back to where it was before
-            location.assign(self.get_iter_at_offset(position))
+                    # `location` becomes invalidated by adding a check/bullet, but it is expected to be there when the
+                    # function ends, so we need to reinitialize it. The default handler places it at the end of the
+                    # inserted text, so that's what we'll do too
+                    location.assign(self.get_iter_at_offset(position + 2))
 
-            next_char = location.copy()
+            # 'location' has usually been shifted by this point, and since it may be used by something else later that
+            # expects the new location, we need to use a different iter
+            start = self.get_iter_at_offset(position)
+
+            next_char = start.copy()
             next_char.forward_char()
 
             # check tag toggles
@@ -647,18 +657,18 @@ class NoteBuffer(Gtk.TextBuffer):
                         font_scale_changed = True
                         for scale_tag_name in FONT_SCALES:
                             if scale_tag_name != 'normal':
-                                action = CompositeAction(action, self.strip_tag(scale_tag_name, location, next_char))
+                                action = CompositeAction(action, self.strip_tag(scale_tag_name, start, next_char))
 
-                    prev = location.copy()
+                    prev = start.copy()
                     if tag_name == 'normal':
                         # the 'normal' tag doesn't really exist but we still use it for removing other font scale tags
                         pass
                     elif prev.backward_char() & prev.has_tag(self.get_tag_table().lookup(tag_name)):
-                        action = CompositeAction(action, self.strip_tag(tag_name, location, next_char))
+                        action = CompositeAction(action, self.strip_tag(tag_name, start, next_char))
                     else:
-                        action = CompositeAction(action, self.add_tag(tag_name, location, next_char))
+                        action = CompositeAction(action, self.add_tag(tag_name, start, next_char))
 
-                for tag in location.get_toggled_tags(False):
+                for tag in start.get_toggled_tags(False):
                     tag_name = tag.props.name
                     if tag_name is None:
                         continue
@@ -667,21 +677,26 @@ class NoteBuffer(Gtk.TextBuffer):
                     if text == '\n' and tag_name == 'header':
                         continue
 
+                    # don't continue links across link-ending characters
+                    if tag_name == 'link' and text in LINK_ENDERS:
+                        continue
+
                     # don't continue the tag if it was toggled off
                     # also don't continue if it's a font scale tag and another was just applied
                     if tag_name not in self.tag_toggles and (tag_name not in FONT_SCALES or not font_scale_changed):
-                        action = CompositeAction(action, self.add_tag(tag_name, location, next_char))
+                        action = CompositeAction(action, self.add_tag(tag_name, start, next_char))
 
                 self.tag_toggles = []
 
             if not self.props.can_undo or not self.undo_actions[-1].maybe_join(action):
                 self.add_undo_action(action)
 
-        if text in ['\n', '\t', ' ', '.', ',', ';', ':']:
-            pre_text = self.get_slice(self.get_start_iter(), location, True)
+        # check for a link if text is a link-ending character
+        if text in LINK_ENDERS:
+            pre_text = self.get_slice(self.get_start_iter(), start, True)
             match = get_url_start(pre_text)
             if match:
-                self.add_undo_action(self.add_tag('link', self.get_iter_at_offset(match.start()), location))
+                self.add_undo_action(self.add_tag('link', self.get_iter_at_offset(match.start()), start))
 
     def on_delete(self, buffer, start, end):
         if self.internal_action_count:
